import warnings

from django.conf import settings
from django.db import DEFAULT_DB_ALIAS, router, connections
from django.utils.module_loading import import_string


def get_migration_class_from_engine(engine):
    backend_name = engine.split('.')[-1]
    # This backend was renamed in Django 1.9.
    if backend_name == 'postgresql_psycopg2':
        backend_name = 'postgresql'
    path = f'django_comment_migrate.backends.{backend_name}.CommentMigration'

    return import_string(path)


def migrate_help_text_to_database(app_config,
                                  using=DEFAULT_DB_ALIAS, **kwargs):
    engine = settings.DATABASES[using]['ENGINE']

    app_label = app_config.label
    if not router.allow_migrate(using, app_label):
        return
    app_models = app_config.get_models()
    try:
        migration_class = get_migration_class_from_engine(engine)
    except ImportError:
        warnings.warn(f'{engine} is not supported by this comment migration'
                      f' backend.')
    else:
        for model in app_models:
            executor = migration_class(connection=connections[using],
                                       model=model)
            executor.execute()



from psycopg2 import sql

from django_comment_migrate.backends.base import BaseCommentMigration
from django_comment_migrate.utils import get_field_comment


class CommentMigration(BaseCommentMigration):
    comment_sql = sql.SQL("COMMENT ON COLUMN {}.{} IS %s")

    def comments_sql(self):
        db_table = self.model._meta.db_table
        changes = []
        for field in self.model._meta.fields:
            comment = get_field_comment(field)
            if comment:
                comment_sql = self.comment_sql.format(
                    sql.Identifier(db_table),
                    sql.Identifier(field.column)
                )
                changes.append(
                    (
                        comment_sql,
                        [comment]
                    )
                )
        if changes:
            return changes



from django.apps import AppConfig
from django.db.models.signals import post_migrate

from django_comment_migrate.db_comments import migrate_help_text_to_database


class DjangoCommentMigrationConfig(AppConfig):
    name = "django_comment_migrate"

    def ready(self):
        post_migrate.connect(migrate_help_text_to_database)



from setuptools import setup

readme = open('README.rst', encoding='utf-8').read()

setup(
    name='django-comment-migrate',
    version='0.0.5',
    description="""An app that provides Django model comment migration""",
    long_description=readme,
    author='starryrbs',
    author_email='1322096624@qq.com',
    url='https://github.com/starryrbs/django-comment-migrate.git',
    keywords='django-comment-migrate',
    packages=['django_comment_migrate'],
    include_package_data=True,
    zip_safe=False,
    license='MIT',
    install_requires=['django>=2.2'],
    python_requires='>=3.5',
)
